@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2008-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2008-2013 -- leonerd@leonerd.org.uk
 
 package IO::Async;
 
@@ -12,7 +12,7 @@ use warnings;
 # It is provided simply to keep CPAN happy:
 #   cpan -i IO::Async
 
-our $VERSION = '0.51_002';
+our $VERSION = '0.60_001';
 
 =head1 NAME
 
@@ -202,23 +202,33 @@ L<IO::Async::Channel> objects. It can be used to solve any sort of problem
 involving keeping a possibly-stateful co-routine running alongside the rest of
 an asynchronous program.
 
-=head2 Tasks
+=head2 Futures
 
-A Task object represents a single outstanding action that is yet to complete,
-such as a name resolution operation or a socket connection. It stands in
-contrast to a C<IO::Async::Notifier>, which is an object that represents an
-ongoing source of activity, such as a readable filehandle of bytes or a POSIX
-signal.
+An L<IO::Async::Future> object represents a single outstanding action that is
+yet to complete, such as a name resolution operation or a socket connection.
+It stands in contrast to a C<IO::Async::Notifier>, which is an object that
+represents an ongoing source of activity, such as a readable filehandle of
+bytes or a POSIX signal.
 
-Tasks are a recent addition to the C<IO::Async> API and details are still
-subject to change and experimentation. Currently a Task object is an instance
-of L<CPS::Future>, but it is possible this may change in future (pardon the
-pun ;) ) to being some class within the C<IO::Async::> namespace, and having
-a reference to the containing Loop object.
+Futures are a recent addition to the C<IO::Async> API and details are still
+subject to change and experimentation.
 
-In general, methods that support Tasks return a new Task object to represent
-the outstanding operation. If callback functions are supplied as well, these
-will be fired in addition to the Task object becoming ready.
+In general, methods that support Futures return a new Future object to
+represent the outstanding operation. If callback functions are supplied as
+well, these will be fired in addition to the Future object becoming ready. Any
+failures that are reported will, in general, use the same conventions for the
+Future's C<fail> arguments to relate it to the legacy C<on_error>-style
+callbacks.
+
+ $on_NAME_error->( $message, @argmuents )
+
+ $f->fail( $message, NAME, @arguments )
+
+where C<$message> is a message intended for humans to read (so that this is
+the message displayed by C<< $f->get >> if the failure is not otherwise
+caught), C<NAME> is the name of the failing operation. If the failure is due
+to a failed system call, the value of C<$!> will be the final argument. The
+message should not end with a linefeed.
 
 =head2 Networking
 
@@ -246,18 +256,8 @@ some of the potentially-useful parts or features currently missing are:
 
 =item *
 
-An L<IO::Async::Loop> subclass to perform integration with L<Event>. Consider
-further ideas on Solaris' I<ports>, BSD's I<Kevents> and anything that might
-be useful on Win32.
-
-=item *
-
-A consideration on how to provide per-OS versions of the utility classes. For
-example, Win32 would probably need an extensively-different C<ChildManager>,
-or OSes may have specific ways to perform asynchronous name resolution
-operations better than the generic C<IO::Async::Function> approach. This
-should be easier to implement now that the L<IO::Async::Loop> magic
-constructor looks for OS-specific subclasses first.
+Consider further ideas on Solaris' I<ports>, BSD's I<Kevents> and anything that
+might be useful on Win32.
 
 =item *
 
@@ -266,19 +266,17 @@ C<IO::Async::Object>, based on C<IO::Async::Routine>.
 
 =item *
 
-Build some sort of future-like system, possibly called C<IO::Async::Task> to
-represent one-shot events like name resolver lookups, socket connects, etc..
-Replace uses of C<Async::MergePoint> with it, removing a dependency.
-Use C<CPS::Future>.
-
-=item *
-
 C<IO::Async::Protocol::Datagram>
 
 =item *
 
 Support for watching filesystem entries for change. Extract logic from
-C<IO::Async::FileStream>. Define Loop watch/unwatch method pair.
+C<IO::Async::File> and define a Loop watch/unwatch method pair.
+
+=item *
+
+Define more C<Future>-returning methods. Consider also one-shot Futures on
+things like C<IO::Async::Process> exits, or C<IO::Async::Handle> close.
 
 =back
 
@@ -310,6 +308,10 @@ L<IO::Async::Loop::Epoll> - use IO::Async with epoll on Linux
 
 =item *
 
+L<IO::Async::Loop::Event> - use IO::Async with Event
+
+=item *
+
 L<IO::Async::Loop::EV> - use IO::Async with EV
 
 =item *
