@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2006-2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2006-2013 -- leonerd@leonerd.org.uk
 
 package IO::Async::Handle;
 
@@ -9,12 +9,16 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.51_002';
+our $VERSION = '0.60_001';
 
 use Carp;
 
 use IO::Handle; # give methods to bare IO handles
 
+use Future;
+
+use IO::Async::OS;
+
 =head1 NAME
 
 C<IO::Async::Handle> - event callbacks for a non-blocking file descriptor
@@ -367,7 +371,17 @@ sub close
    my $write_handle = delete $self->{write_handle};
    $write_handle->close if defined $write_handle;
 
+   $self->_closed;
+}
+
+sub _closed
+{
+   my $self = shift;
+
    $self->maybe_invoke_event( on_closed => );
+   if( $self->{close_futures} ) {
+      $_->done for @{ $self->{close_futures} };
+   }
    $self->remove_from_parent;
 }
 
@@ -392,10 +406,7 @@ sub close_read
    my $read_handle = delete $self->{read_handle};
    $read_handle->close if defined $read_handle;
 
-   if( !$self->{write_handle} ) {
-      $self->maybe_invoke_event( on_closed => );
-      $self->remove_from_parent;
-   }
+   $self->_closed if !$self->{write_handle};
 }
 
 sub close_write
@@ -407,10 +418,32 @@ sub close_write
    my $write_handle = delete $self->{write_handle};
    $write_handle->close if defined $write_handle;
 
-   if( !$self->{read_handle} ) {
-      $self->maybe_invoke_event( on_closed => );
-      $self->remove_from_parent;
-   }
+   $self->_closed if !$self->{read_handle};
+}
+
+=head2 $future = $handle->new_close_future
+
+Returns a new L<IO::Async::Future> object which will become done when the
+handle is closed. Cancelling the C<$future> will remove this notification
+ability but will not otherwise affect the C<$handle>.
+
+=cut
+
+sub new_close_future
+{
+   my $self = shift;
+
+   push @{ $self->{close_futures} }, my $future = $self->loop->new_future;
+   $future->on_cancel(
+      $self->_capture_weakself( sub {
+         my $self = shift or return;
+         my $future = shift;
+
+         @{ $self->{close_futures} } = grep { $_ != $future } @{ $self->{close_futures} };
+      })
+   );
+
+   return $future;
 }
 
 =head2 $handle = $handle->read_handle
@@ -522,6 +555,74 @@ sub want_writeready
    }
 }
 
+=head2 $handle->socket( $ai )
+
+Convenient shortcut to creating a socket handle, as given by an addrinfo
+structure, and setting it as the read and write handle for the object.
+
+C<$ai> may be either a C<HASH> or C<ARRAY> reference of the same form as given
+to L<IO::Async::OS>'s C<extract_addrinfo> method.
+
+This method returns nothing if it succeeds, or throws an exception if it
+fails.
+
+=cut
+
+sub socket
+{
+   my $self = shift;
+   my ( $ai ) = @_;
+
+   # TODO: Something about closing the old one?
+
+   my ( $family, $socktype, $protocol ) = IO::Async::OS->extract_addrinfo( $ai );
+
+   my $sock = IO::Async::OS->socket( $family, $socktype, $protocol );
+   $self->set_handle( $sock );
+}
+
+=head2 $handle->bind( $ai )
+
+Convenient shortcut to creating a socket handle and C<bind()>ing it to the
+address as given by an addrinfo structure, and setting it as the read and
+write handle for the object.
+
+C<$ai> may be either a C<HASH> or C<ARRAY> reference of the same form as given
+to L<IO::Async::OS>'s C<extract_addrinfo> method.
+
+This method returns nothing if it succeeds, or throws an exception if it
+fails.
+
+=cut
+
+sub bind
+{
+   my $self = shift;
+   my ( $ai ) = @_;
+
+   $self->socket( $ai );
+   my $addr = ( IO::Async::OS->extract_addrinfo( $ai ) )[3];
+
+   $self->read_handle->bind( $addr ) or croak "Cannot bind - $!";
+}
+
+=head2 $future = $handle->connect( %args )
+
+A convenient wrapper for calling the C<connect> method on the underlying
+L<IO::Async::Loop> object.
+
+=cut
+
+sub connect
+{
+   my $self = shift;
+   my %args = @_;
+
+   my $loop = $self->loop or croak "Cannot ->connect a Handle that is not in a Loop";
+
+   return $self->loop->connect( %args, handle => $self );
+}
+
 =head1 SEE ALSO
 
 =over 4
