@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2013 -- leonerd@leonerd.org.uk
 
 package IO::Async::Loop;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.51_002';
+our $VERSION = '0.60_001';
 
 # When editing this value don't forget to update the docs below
 use constant NEED_API_VERSION => '0.33';
@@ -16,9 +16,17 @@ use constant NEED_API_VERSION => '0.33';
 # Base value but some classes might override
 use constant _CAN_ON_HANGUP => 0;
 
-# Some Loop implementations do not accurately handle sub-second timers.
+# Most Loop implementations do not accurately handle sub-second timers.
 # This only matters for unit tests
-use constant _CAN_SUBSECOND_ACCURATELY => 1;
+use constant _CAN_SUBSECOND_ACCURATELY => 0;
+
+# Does the loop implementation support IO_ASYNC_WATCHDOG?
+use constant _CAN_WATCHDOG => 0;
+
+# Watchdog configuration constants
+use constant WATCHDOG_ENABLE   => $ENV{IO_ASYNC_WATCHDOG};
+use constant WATCHDOG_INTERVAL => $ENV{IO_ASYNC_WATCHDOG_INTERVAL} || 10;
+use constant WATCHDOG_SIGABRT  => $ENV{IO_ASYNC_WATCHDOG_SIGABRT};
 
 use Carp;
 
@@ -26,9 +34,12 @@ use IO::Socket (); # empty import
 use Time::HiRes qw(); # empty import
 use POSIX qw( _exit WNOHANG );
 use Scalar::Util qw( refaddr );
+use Socket qw( SO_REUSEADDR AF_INET6 IPPROTO_IPV6 IPV6_V6ONLY );
 
 use IO::Async::OS;
 
+use constant HAVE_SIGNALS => IO::Async::OS->HAVE_SIGNALS;
+
 # Never sleep for more than 1 second if a signal proxy is registered, to avoid
 # a borderline race condition.
 # There is a race condition in perl involving signals interacting with XS code
@@ -39,6 +50,10 @@ use IO::Async::OS;
 # run when the XS function returns. 
 our $MAX_SIGWAIT_TIME = 1;
 
+# Also, never sleep for more than 1 second if the OS does not support signals
+# and we have child watches registered (so we must use waitpid() polling)
+our $MAX_CHILDWAIT_TIME = 1;
+
 # Maybe our calling program will have a suggested hint of a specific Loop
 # class or list of classes to use
 our $LOOP;
@@ -48,6 +63,19 @@ our $LOOP;
 # magic constructor
 our $LOOP_NO_OS;
 
+# SIGALRM handler for watchdog
+$SIG{ALRM} = sub {
+   # There are two extra frames here; this one and the signal handler itself
+   local $Carp::CarpLevel = $Carp::CarpLevel + 2;
+   if( WATCHDOG_SIGABRT ) {
+      print STDERR Carp::longmess( "Watchdog timeout" );
+      kill ABRT => $$;
+   }
+   else {
+      Carp::confess( "Watchdog timeout" );
+   }
+} if WATCHDOG_ENABLE;
+
 =head1 NAME
 
 C<IO::Async::Loop> - core loop of the C<IO::Async> framework
@@ -117,6 +145,9 @@ sub __new
    $class->API_VERSION >= NEED_API_VERSION or
       die "$class is too old for IO::Async $VERSION; we need API version >= ".NEED_API_VERSION.", it provides ".$class->API_VERSION."\n";
 
+   WATCHDOG_ENABLE and !$class->_CAN_WATCHDOG and
+      warn "$class cannot implement IO_ASYNC_WATCHDOG\n";
+
    my $self = bless {
       notifiers    => {}, # {nkey} = notifier
       iowatches    => {}, # {fd} = [ $on_read_ready, $on_write_ready, $on_hangup ]
@@ -139,6 +170,10 @@ sub __new
       die "$class should overload both ->enqueue_timer and ->cancel_timer, or neither";
    }
 
+   if( $old_timer ) {
+      warnings::warnif( deprecated => "Enabling old_timer workaround for old loop class " . $class );
+   }
+
    $self->{old_timer} = $old_timer;
 
    return $self;
@@ -250,7 +285,6 @@ sub new
 sub really_new
 {
    shift;  # We're going to ignore the class name actually given
-
    my $self;
 
    my @candidates;
@@ -269,8 +303,17 @@ sub really_new
 
    $self = __try_new( "IO::Async::Loop::$^O" ) and return $self unless $LOOP_NO_OS;
 
-   $self = __try_new( "IO::Async::Loop::Poll" )   and return $self;
-   $self = __try_new( "IO::Async::Loop::Select" ) and return $self;
+   return IO::Async::Loop->new_builtin;
+}
+
+sub new_builtin
+{
+   shift;
+   my $self;
+
+   foreach my $class ( IO::Async::OS->LOOP_BUILTIN_CLASSES ) {
+      $self = __try_new( "IO::Async::Loop::$class" ) and return $self;
+   }
 
    croak "Cannot find a suitable candidate class";
 }
@@ -479,41 +522,53 @@ sub loop_stop
    $self->stop;
 }
 
-#########
-# Tasks #
-#########
+###########
+# Futures #
+###########
 
-=head1 TASK SUPPORT
+=head1 FUTURE SUPPORT
 
-To support the use of Task objects in semi-synchronous programs, the following
-methods all block until some condition involving tasks is satisifed.
+The following methods relate to L<IO::Async::Future> objects.
 
 =cut
 
-=head2 $loop->await( $task )
+=head2 $future = $loop->new_future
 
-Blocks until the given task is ready, as indicated by its C<is_ready> method.
-As a convenience it returns the task, to simplify code:
+Returns a new C<IO::Async::Future> instance with a reference to the Loop.
 
- my @result = $loop->await( $task )->get;
+=cut
+
+sub new_future
+{
+   my $self = shift;
+   require IO::Async::Future;
+   return IO::Async::Future->new( $self );
+}
+
+=head2 $loop->await( $future )
+
+Blocks until the given future is ready, as indicated by its C<is_ready> method.
+As a convenience it returns the future, to simplify code:
+
+ my @result = $loop->await( $future )->get;
 
 =cut
 
 sub await
 {
    my $self = shift;
-   my ( $task ) = @_;
+   my ( $future ) = @_;
 
-   $self->loop_once until $task->is_ready;
+   $self->loop_once until $future->is_ready;
 
-   return $task;
+   return $future;
 }
 
-=head2 $loop->await_all( @tasks )
+=head2 $loop->await_all( @futures )
 
-Blocks until all the given tasks are ready, as indicated by the C<is_ready>
-method. Equivalent to calling C<await> on a C<< CPS::Future->wait_all >>
-except that it doesn't create the surrounding future object.
+Blocks until all the given futures are ready, as indicated by the C<is_ready>
+method. Equivalent to calling C<await> on a C<< Future->wait_all >> except
+that it doesn't create the surrounding future object.
 
 =cut
 
@@ -522,9 +577,57 @@ sub _all_ready { $_->is_ready or return 0 for @_; return 1  }
 sub await_all
 {
    my $self = shift;
-   my @tasks = @_;
+   my @futures = @_;
 
-   $self->loop_once until _all_ready @tasks;
+   $self->loop_once until _all_ready @futures;
+}
+
+=head2 $future = $loop->delay_future( %args )
+
+Returns a new C<IO::Async::Future> instance which will become done at a given
+point in time. The C<%args> should contain an C<at> or C<after> key as per the
+C<watch_time> method. The returned future may be cancelled to cancel the
+timer. At the alloted time the future will succeed with an empty result list.
+
+=cut
+
+sub delay_future
+{
+   my $self = shift;
+   my %args = @_;
+
+   my $future = $self->new_future;
+   my $id = $self->watch_time( %args,
+      code => sub { $future->done },
+   );
+
+   $future->on_cancel( sub { shift->loop->unwatch_time( $id ) } );
+
+   return $future;
+}
+
+=head2 $future = $loop->timeout_future( %args )
+
+Returns a new C<IO::Async::Future> instance which will fail at a given point
+in time. The C<%args> should contain an C<at> or C<after> key as per the
+C<watch_time> method. The returned future may be cancelled to cancel the
+timer. At the alloted time, the future will fail with the string C<"Timeout">.
+
+=cut
+
+sub timeout_future
+{
+   my $self = shift;
+   my %args = @_;
+
+   my $future = $self->new_future;
+   my $id = $self->watch_time( %args,
+      code => sub { $future->fail( "Timeout" ) },
+   );
+
+   $future->on_cancel( sub { shift->loop->unwatch_time( $id ) } );
+
+   return $future;
 }
 
 ############
@@ -597,6 +700,8 @@ sub attach_signal
    my $self = shift;
    my ( $signal, $code ) = @_;
 
+   HAVE_SIGNALS or croak "This OS cannot ->attach_signal";
+
    if( $signal eq "CHLD" ) {
       # We make special exception to allow $self->watch_child to do this
       caller eq "IO::Async::Loop" or
@@ -642,6 +747,8 @@ sub detach_signal
    my $self = shift;
    my ( $signal, $id ) = @_;
 
+   HAVE_SIGNALS or croak "This OS cannot ->detach_signal";
+
    # Can't use grep because we have to preserve the addresses
    my $attaches = $self->{sigattaches}->{$signal} or return;
 
@@ -682,14 +789,6 @@ sub later
    return $self->watch_idle( when => 'later', code => $code );
 }
 
-# undocumented, to be removed soon
-sub detach_child
-{
-   my $self = shift;
-   warnings::warnif( deprecated => "Loop->detach_child is deprecated; use ->fork instead" );
-   $self->fork( @_ );
-}
-
 =head2 $loop->spawn_child( %params )
 
 This method creates a new child process to run a given code block or command.
@@ -953,15 +1052,212 @@ sub resolve
    $self->resolver->resolve( %params );
 }
 
-=head2 $loop->connect( %params )
+=head2 $future = $loop->connect( %params )
 
-This method performs a non-blocking connect operation. It uses an
-internally-stored C<IO::Async::Connector> object. For more detail, see the
-C<connect> method on the L<IO::Async::Connector> class.
+This method performs a non-blocking connection to a given address or set of
+addresses, returning a L<IO::Async::Future> which represents the operation. On
+completion, the future will yield the connected socket handle, or the given
+L<IO::Async::Handle> object.
+
+There are two modes of operation. Firstly, a list of addresses can be provided
+which will be tried in turn. Alternatively as a convenience, if a host and
+service name are provided instead of a list of addresses, these will be
+resolved using the underlying loop's C<resolve> method into the list of
+addresses.
+
+When attempting to connect to any among a list of addresses, there may be
+failures among the first attempts, before a valid connection is made. For
+example, the resolver may have returned some IPv6 addresses, but only IPv4
+routes are valid on the system. In this case, the first C<connect(2)> syscall
+will fail. This isn't yet a fatal error, if there are more addresses to try,
+perhaps some IPv4 ones.
+
+For this reason, it is possible that the operation eventually succeeds even
+though some system calls initially fail. To be aware of individual failures,
+the optional C<on_fail> callback can be used. This will be invoked on each
+individual C<socket(2)> or C<connect(2)> failure, which may be useful for
+debugging or logging.
+
+Because this module simply uses the C<getaddrinfo> resolver, it will be fully
+IPv6-aware if the underlying platform's resolver is. This allows programs to
+be fully IPv6-capable.
+
+In plain address mode, the C<%params> hash takes the following keys:
+
+=over 8
+
+=item addrs => ARRAY
+
+Reference to an array of (possibly-multiple) address structures to attempt to
+connect to. Each should be in the layout described for C<addr>. Such a layout
+is returned by the C<getaddrinfo> named resolver.
+
+=item addr => HASH or ARRAY
+
+Shortcut for passing a single address to connect to; it may be passed directly
+with this key, instead of in another array on its own. This should be in a
+format recognised by L<IO::Async::OS>'s C<extract_addrinfo> method.
+
+This example shows how to use the C<Socket> functions to construct one for TCP
+port 8001 on address 10.0.0.1:
+
+ $loop->connect(
+    addr => {
+       family   => "inet",
+       socktype => "stream",
+       port     => 8001,
+       ip       => "10.0.0.1",
+    },
+    ...
+ );
+
+This example shows another way to connect to a UNIX socket at F<echo.sock>.
+
+ $loop->connect(
+    addr => {
+       family   => "unix",
+       socktype => "stream",
+       path     => "echo.sock",
+    },
+    ...
+ );
+
+=item local_addrs => ARRAY
+
+=item local_addr => HASH or ARRAY
+
+Optional. Similar to the C<addrs> or C<addr> parameters, these specify a local
+address or set of addresses to C<bind(2)> the socket to before
+C<connect(2)>ing it.
+
+=back
+
+When performing the resolution step too, the C<addrs> or C<addr> keys are
+ignored, and instead the following keys are taken:
+
+=over 8
+
+=item host => STRING
+
+=item service => STRING
+
+The hostname and service name to connect to.
+
+=item local_host => STRING
+
+=item local_service => STRING
+
+Optional. The hostname and/or service name to C<bind(2)> the socket to locally
+before connecting to the peer.
+
+=item family => INT
+
+=item socktype => INT
+
+=item protocol => INT
+
+=item flags => INT
+
+Optional. Other arguments to pass along with C<host> and C<service> to the
+C<getaddrinfo> call.
+
+=item socktype => STRING
+
+Optionally may instead be one of the values C<'stream'>, C<'dgram'> or
+C<'raw'> to stand for C<SOCK_STREAM>, C<SOCK_DGRAM> or C<SOCK_RAW>. This
+utility is provided to allow the caller to avoid a separate C<use Socket> only
+for importing these constants.
+
+=back
+
+It is necessary to pass the C<socktype> hint to the resolver when resolving
+the host/service names into an address, as some OS's C<getaddrinfo> functions
+require this hint. A warning is emitted if neither C<socktype> nor C<protocol>
+hint is defined when performing a C<getaddrinfo> lookup. To avoid this warning
+while still specifying no particular C<socktype> hint (perhaps to invoke some
+OS-specific behaviour), pass C<0> as the C<socktype> value.
+
+In either case, it also accepts the following arguments:
+
+=over 8
+
+=item handle => IO::Async::Handle
+
+Optional. If given a L<IO::Async::Handle> object or a subclass (such as
+L<IO::Async::Stream> or L<IO::Async::Socket> its handle will be set to the
+newly-connected socket on success, and that handle used as the result of the
+future instead.
+
+=item on_fail => CODE
+
+Optional. After an individual C<socket(2)> or C<connect(2)> syscall has failed,
+this callback is invoked to inform of the error. It is passed the name of the
+syscall that failed, the arguments that were passed to it, and the error it
+generated. I.e.
+
+ $on_fail->( "socket", $family, $socktype, $protocol, $! );
+
+ $on_fail->( "bind", $sock, $address, $! );
+
+ $on_fail->( "connect", $sock, $address, $! );
+
+Because of the "try all" nature when given a list of multiple addresses, this
+callback may be invoked multiple times, even before an eventual success.
+
+=back
 
 This method accepts an C<extensions> parameter; see the C<EXTENSIONS> section
 below.
 
+=head2 $loop->connect( %params )
+
+When not returning a future, additional parameters can be given containing the
+continuations to invoke on success or failure.
+
+=over 8
+
+=item on_connected => CODE
+
+A continuation that is invoked on a successful C<connect(2)> call to a valid
+socket. It will be passed the connected socket handle, as an C<IO::Socket>
+object.
+
+ $on_connected->( $handle )
+
+=item on_stream => CODE
+
+An alternative to C<on_connected>, a continuation that is passed an instance
+of L<IO::Async::Stream> when the socket is connected. This is provided as a
+convenience for the common case that a Stream object is required as the
+transport for a Protocol object.
+
+ $on_stream->( $stream )
+
+=item on_socket => CODE
+
+Similar to C<on_stream>, but constructs an instance of L<IO::Async::Socket>.
+This is most useful for C<SOCK_DGRAM> or C<SOCK_RAW> sockets.
+
+ $on_socket->( $socket )
+
+=item on_connect_error => CODE
+
+A continuation that is invoked after all of the addresses have been tried, and
+none of them succeeded. It will be passed the most significant error that
+occurred, and the name of the operation it occurred in. Errors from the
+C<connect(2)> syscall are considered most significant, then C<bind(2)>, then
+finally C<socket(2)>.
+
+ $on_connect_error->( $syscall, $! )
+
+=item on_resolve_error => CODE
+
+A continuation that is invoked when the name resolution attempt fails. This is
+invoked in the same way as the C<on_error> continuation for the C<resolve>
+method.
+
+=back
+
 =cut
 
 sub connect
@@ -978,36 +1274,204 @@ sub connect
 
       $self->can( $method ) or croak "Extension method '$method' is not available";
 
-      $self->$method(
+      return $self->$method(
          %params,
          ( @others ? ( extensions => \@others ) : () ),
       );
-      return;
    }
 
-   my $connector = $self->{connector} ||= $self->__new_feature( "IO::Async::Connector" );
+   my $handle = $params{handle};
 
-   $connector->connect( %params );
+   my $on_done;
+   # Legacy callbacks
+   if( my $on_connected = delete $params{on_connected} ) {
+      $on_done = $on_connected;
+   }
+   elsif( my $on_stream = delete $params{on_stream} ) {
+      defined $handle and croak "Cannot pass 'on_stream' with a handle object as well";
+
+      require IO::Async::Stream;
+      # TODO: It doesn't make sense to put a SOCK_DGRAM in an
+      # IO::Async::Stream but currently we don't detect this
+      $handle = IO::Async::Stream->new;
+      $on_done = $on_stream;
+   }
+   elsif( my $on_socket = delete $params{on_socket} ) {
+      defined $handle and croak "Cannot pass 'on_socket' with a handle object as well";
+
+      require IO::Async::Socket;
+      $handle = IO::Async::Socket->new;
+      $on_done = $on_socket;
+   }
+   elsif( !defined wantarray ) {
+      croak "Expected 'on_connected' or 'on_stream' callback or to return a Future";
+   }
+
+   my $on_connect_error;
+   if( $on_connect_error = $params{on_connect_error} ) {
+      # OK
+   }
+   elsif( !defined wantarray ) {
+      croak "Expected 'on_connect_error' callback";
+   }
+
+   my $on_resolve_error;
+   if( $on_resolve_error = $params{on_resolve_error} ) {
+      # OK
+   }
+   elsif( !defined wantarray and exists $params{host} || exists $params{local_host} ) {
+      croak "Expected 'on_resolve_error' callback or to return a Future";
+   }
+
+   my $connector = $self->{connector} ||= $self->__new_feature( "IO::Async::Internals::Connector" );
+
+   my $future = $connector->connect( %params );
+
+   $future = $future->then( sub {
+      $handle->set_handle( shift );
+      return Future->new->done( $handle )
+   }) if $handle;
+
+   $future->on_done( $on_done ) if $on_done;
+   $future->on_fail( sub {
+      $on_connect_error->( @_[2,3] ) if $on_connect_error and $_[1] eq "connect";
+      $on_resolve_error->( $_[2] )   if $on_resolve_error and $_[1] eq "resolve";
+   } );
+
+   return $future;
 }
 
-=head2 $loop->listen( %params )
+=head2 $loop->listen( %params ) ==> $listener
 
-This method sets up a listening socket. It creates an instance of
-L<IO::Async::Listener> and adds it to the Loop.
+This method sets up a listening socket and arranges for an acceptor callback
+to be invoked each time a new connection is accepted on the socket. Internally
+it creates an instance of L<IO::Async::Listener> and adds it to the Loop if
+not given one in the arguments.
 
-Most parameters given to this method are passed into the constructed Listener
-object's C<listen> method. In addition, the following arguments are also
-recognised directly:
+Addresses may be given directly, or they may be looked up using the system's
+name resolver, or a socket handle may be given directly.
+
+If multiple addresses are given, or resolved from the service and hostname,
+then each will be attempted in turn until one succeeds.
+
+In named resolver mode, the C<%params> hash takes the following keys:
 
 =over 8
 
-=item on_listen => CODE
+=item service => STRING
 
-Optional. A callback that is invoked when the listening socket is ready.
-Typically this would be used in the name resolver case, in order to inspect
-the socket's sockname address, or otherwise inspect the filehandle.
+The service name to listen on.
 
- $on_listen->( $socket )
+=item host => STRING
+
+The hostname to listen on. Optional. Will listen on all addresses if not
+supplied.
+
+=item family => INT
+
+=item socktype => INT
+
+=item protocol => INT
+
+=item flags => INT
+
+Optional. Other arguments to pass along with C<host> and C<service> to the
+C<getaddrinfo> call.
+
+=item socktype => STRING
+
+Optionally may instead be one of the values C<'stream'>, C<'dgram'> or
+C<'raw'> to stand for C<SOCK_STREAM>, C<SOCK_DGRAM> or C<SOCK_RAW>. This
+utility is provided to allow the caller to avoid a separate C<use Socket> only
+for importing these constants.
+
+=back
+
+It is necessary to pass the C<socktype> hint to the resolver when resolving
+the host/service names into an address, as some OS's C<getaddrinfo> functions
+require this hint. A warning is emitted if neither C<socktype> nor C<protocol>
+hint is defined when performing a C<getaddrinfo> lookup. To avoid this warning
+while still specifying no particular C<socktype> hint (perhaps to invoke some
+OS-specific behaviour), pass C<0> as the C<socktype> value.
+
+In plain address mode, the C<%params> hash takes the following keys:
+
+=over 8
+
+=item addrs => ARRAY
+
+Reference to an array of (possibly-multiple) address structures to attempt to
+listen on. Each should be in the layout described for C<addr>. Such a layout
+is returned by the C<getaddrinfo> named resolver.
+
+=item addr => ARRAY
+
+Shortcut for passing a single address to listen on; it may be passed directly
+with this key, instead of in another array of its own. This should be in a
+format recognised by L<IO::Async::OS>'s C<extract_addrinfo> method. See also
+the C<EXAMPLES> section.
+
+=back
+
+In direct socket handle mode, the following keys are taken:
+
+=over 8
+
+=item handle => IO
+
+The listening socket handle.
+
+=back
+
+In either case, the following keys are also taken:
+
+=over 8
+
+=item on_fail => CODE
+
+Optional. A callback that is invoked if a syscall fails while attempting to
+create a listening sockets. It is passed the name of the syscall that failed,
+the arguments that were passed to it, and the error generated. I.e.
+
+ $on_fail->( "socket", $family, $socktype, $protocol, $! );
+
+ $on_fail->( "sockopt", $sock, $optname, $optval, $! );
+
+ $on_fail->( "bind", $sock, $address, $! );
+
+ $on_fail->( "listen", $sock, $queuesize, $! );
+
+=item queuesize => INT
+
+Optional. The queue size to pass to the C<listen(2)> calls. If not supplied,
+then 3 will be given instead.
+
+=item reuseaddr => BOOL
+
+Optional. If true or not supplied then the C<SO_REUSEADDR> socket option will
+be set. To prevent this, pass a false value such as 0.
+
+=item v6only => BOOL
+
+Optional. If defined, sets or clears the C<IPV6_V6ONLY> socket option on
+C<PF_INET6> sockets. This option disables the ability of C<PF_INET6> socket to
+accept connections from C<AF_INET> addresses. Not all operating systems allow
+this option to be disabled.
+
+=back
+
+An alternative which gives more control over the listener, is to create the
+C<IO::Async::Listener> object directly and add it explicitly to the Loop.
+
+This method accepts an C<extensions> parameter; see the C<EXTENSIONS> section
+below.
+
+=head2 $loop->listen( %params )
+
+When not returning a future, additional parameters can be given containing the
+continuations to invoke on success or failure.
+
+=over 8
 
 =item on_notifier => CODE
 
@@ -1019,14 +1483,30 @@ receive connections. The callback is passed the Listener object itself.
 If this callback is required, it may instead be better to construct the
 Listener object directly.
 
+=item on_listen => CODE
+
+Optional. A callback that is invoked when the listening socket is ready.
+Typically this would be used in the name resolver case, in order to inspect
+the socket's sockname address, or otherwise inspect the filehandle.
+
+ $on_listen->( $socket )
+
+=item on_listen_error => CODE
+
+A continuation this is invoked after all of the addresses have been tried, and
+none of them succeeded. It will be passed the most significant error that
+occurred, and the name of the operation it occurred in. Errors from the
+C<listen(2)> syscall are considered most significant, then C<bind(2)>, then
+C<sockopt(2)>, then finally C<socket(2)>.
+
+=item on_resolve_error => CODE
+
+A continuation that is invoked when the name resolution attempt fails. This is
+invoked in the same way as the C<on_error> continuation for the C<resolve>
+method.
+
 =back
 
-An alternative which gives more control over the listener, is to create the
-C<IO::Async::Listener> object directly and add it explicitly to the Loop.
-
-This method accepts an C<extensions> parameter; see the C<EXTENSIONS> section
-below.
-
 =cut
 
 sub listen
@@ -1034,71 +1514,196 @@ sub listen
    my $self = shift;
    my ( %params ) = @_;
 
+   my $remove_on_error;
+   my $listener = $params{listener} ||= do {
+      $remove_on_error++;
+
+      require IO::Async::Listener;
+
+      # Our wrappings of these don't want $listener
+      my %listenerparams;
+      for (qw( on_accept on_stream on_socket )) {
+         next unless exists $params{$_};
+         croak "Cannot ->listen with '$_' and 'listener'" if $params{listener};
+
+         my $code = delete $params{$_};
+         $listenerparams{$_} = sub {
+            shift;
+            goto &$code;
+         };
+      }
+
+      my $listener = IO::Async::Listener->new( %listenerparams );
+      $self->add( $listener );
+      $listener
+   };
+
    my $extensions;
    if( $extensions = delete $params{extensions} and @$extensions ) {
       my ( $ext, @others ) = @$extensions;
 
+      # We happen to know we break older IO::Async::SSL
+      if( $ext eq "SSL" and $IO::Async::SSL::VERSION < '0.12001' ) {
+         croak "IO::Async::SSL version too old; need at least 0.12_001; found $IO::Async::SSL::VERSION";
+      }
+
       my $method = "${ext}_listen";
       # TODO: Try to 'require IO::Async::$ext'
 
       $self->can( $method ) or croak "Extension method '$method' is not available";
 
-      $self->$method(
+      my $f = $self->$method(
          %params,
          ( @others ? ( extensions => \@others ) : () ),
       );
-      return;
+      $f->on_fail( sub { $self->remove( $listener ) } ) if $remove_on_error;
+
+      return $f;
    }
 
-   require IO::Async::Listener;
+   my $on_notifier = delete $params{on_notifier}; # optional
 
-   my $on_notifier = delete $params{on_notifier};
+   my $on_listen_error  = delete $params{on_listen_error};
+   my $on_resolve_error = delete $params{on_resolve_error};
 
-   my %listenerparams;
+   # Shortcut
+   if( $params{addr} and not $params{addrs} ) {
+      $params{addrs} = [ delete $params{addr} ];
+   }
 
+   my $f;
    if( my $handle = delete $params{handle} ) {
-      $listenerparams{handle} = $handle;
+      $f = $self->_listen_handle( $listener, $handle, %params );
    }
-
-   # Our wrappings of these don't want $self
-   for (qw( on_accept on_stream on_socket )) {
-      next unless exists $params{$_};
-      my $code = delete $params{$_};
-      $listenerparams{$_} = sub {
-         shift;
-         goto &$code;
-      };
+   elsif( my $addrs = delete $params{addrs} ) {
+      $on_listen_error or defined wantarray or
+         croak "Expected 'on_listen_error' or to return a Future";
+      $f = $self->_listen_addrs( $listener, $addrs, %params );
    }
-
-   my $listener = IO::Async::Listener->new( %listenerparams );
-
-   $self->add( $listener );
-
-   if( $listener->is_listening ) {
-      $on_notifier->( $listener ) if $on_notifier;
+   elsif( defined $params{service} ) {
+      $on_listen_error or defined wantarray or
+         croak "Expected 'on_listen_error' or to return a Future";
+      $on_resolve_error or defined wantarray or
+         croak "Expected 'on_resolve_error' or to return a Future";
+      $f = $self->_listen_hostservice( $listener, delete $params{host}, delete $params{service}, %params );
    }
    else {
-      my $on_listen = delete $params{on_listen};
-      $listener->listen( 
-         %params,
-         on_listen => sub {
-            my ( $sock ) = @_;
-            $on_listen->( $listener->read_handle ) if $on_listen;
-            $on_notifier->( $listener ) if $on_notifier;
-         },
-         on_listen_error => sub {
-            $self->remove( $listener );
-            $params{on_listen_error}->( @_ ) if $params{on_listen_error};
-         },
-         on_resolve_error => sub {
-            $self->remove( $listener );
-            $params{on_resolve_error}->( @_ ) if $params{on_resolve_error};
-         },
-      );
+      croak "Expected either 'service' or 'addrs' or 'addr' arguments";
    }
 
-   # Unit testing needs this
-   return $listener;
+   $f->on_done( $on_notifier ) if $on_notifier;
+   if( my $on_listen = $params{on_listen} ) {
+      $f->on_done( sub { $on_listen->( shift->read_handle ) } );
+   }
+   $f->on_fail( sub {
+      my ( $message, $how, @rest ) = @_;
+      $on_listen_error->( @rest )  if $on_listen_error  and $how eq "listen";
+      $on_resolve_error->( @rest ) if $on_resolve_error and $how eq "resolve";
+   });
+   $f->on_fail( sub { $self->remove( $listener ) } ) if $remove_on_error;
+
+   return $f;
+}
+
+sub _listen_handle
+{
+   my $self = shift;
+   my ( $listener, $handle, %params ) = @_;
+
+   $listener->configure( handle => $handle );
+   return $self->new_future->done( $listener );
+}
+
+sub _listen_addrs
+{
+   my $self = shift;
+   my ( $listener, $addrs, %params ) = @_;
+
+   my $queuesize = $params{queuesize} || 3;
+
+   my $on_fail = $params{on_fail};
+   !defined $on_fail or ref $on_fail or croak "Expected 'on_fail' to be a reference";
+
+   my $reuseaddr = 1;
+   $reuseaddr = 0 if defined $params{reuseaddr} and not $params{reuseaddr};
+
+   my $v6only = $params{v6only};
+
+   my ( $listenerr, $binderr, $sockopterr, $socketerr );
+
+   foreach my $addr ( @$addrs ) {
+      my ( $family, $socktype, $proto, $address ) = IO::Async::OS->extract_addrinfo( $addr );
+
+      my $sock;
+
+      unless( $sock = IO::Async::OS->socket( $family, $socktype, $proto ) ) {
+         $socketerr = $!;
+         $on_fail->( socket => $family, $socktype, $proto, $! ) if $on_fail;
+         next;
+      }
+
+      if( $reuseaddr ) {
+         unless( $sock->sockopt( SO_REUSEADDR, 1 ) ) {
+            $sockopterr = $!;
+            $on_fail->( sockopt => $sock, SO_REUSEADDR, 1, $! ) if $on_fail;
+            next;
+         }
+      }
+
+      if( defined $v6only and $family == AF_INET6 ) {
+         unless( $sock->setsockopt( IPPROTO_IPV6, IPV6_V6ONLY, $v6only ) ) {
+            $sockopterr = $!;
+            $on_fail->( sockopt => $sock, IPV6_V6ONLY, $v6only, $! ) if $on_fail;
+            next;
+         }
+      }
+
+      unless( $sock->bind( $address ) ) {
+         $binderr = $!;
+         $on_fail->( bind => $sock, $address, $! ) if $on_fail;
+         next;
+      }
+
+      unless( $sock->listen( $queuesize ) ) {
+         $listenerr = $!;
+         $on_fail->( listen => $sock, $queuesize, $! ) if $on_fail;
+         next;
+      }
+
+      return $self->_listen_handle( $listener, $sock, %params );
+   }
+
+   my $f = $self->new_future;
+   return $f->fail( "Cannot listen() - $listenerr",      listen => listen  => $listenerr  ) if $listenerr;
+   return $f->fail( "Cannot bind() - $binderr",          listen => bind    => $binderr    ) if $binderr;
+   return $f->fail( "Cannot setsockopt() - $sockopterr", listen => sockopt => $sockopterr ) if $sockopterr;
+   return $f->fail( "Cannot socket() - $socketerr",      listen => socket  => $socketerr  ) if $socketerr;
+   die 'Oops; $loop->listen failed but no error cause was found';
+}
+
+sub _listen_hostservice
+{
+   my $self = shift;
+   my ( $listener, $host, $service, %params ) = @_;
+
+   $host ||= "";
+   defined $service or $service = ""; # might be 0
+
+   my %gai_hints;
+   exists $params{$_} and $gai_hints{$_} = $params{$_} for qw( family socktype protocol flags );
+
+   defined $gai_hints{socktype} or defined $gai_hints{protocol} or
+      carp "Attempting to ->listen without either 'socktype' or 'protocol' hint is not portable";
+
+   $self->resolver->getaddrinfo(
+      host    => $host,
+      service => $service,
+      passive => 1,
+      %gai_hints,
+   )->then( sub {
+      my @addrs = @_;
+      $self->_listen_addrs( $listener, \@addrs, %params );
+   });
 }
 
 =head1 OS ABSTRACTIONS
@@ -1109,26 +1714,14 @@ to give different implementations on that OS.
 
 =cut
 
-=head2 ( $S1, $S2 ) = $loop->socketpair( $family, $socktype, $proto )
-
-=head2 ( $rd, $wr ) = $loop->pipepair
-
-=head2 ( $rdA, $wrA, $rdB, $wrB ) = $loop->pipequad
-
 =head2 $signum = $loop->signame2num( $signame )
 
 Legacy wrappers around L<IO::Async::OS> functions.
 
 =cut
 
-sub socketpair  { shift; IO::Async::OS->socketpair( @_ ) }
-sub pipepair    { shift; IO::Async::OS->pipepair( @_ ) }
-sub pipequad    { shift; IO::Async::OS->pipequad( @_ ) }
 sub signame2num { shift; IO::Async::OS->signame2num( @_ ) }
 
-sub extract_addrinfo { shift; IO::Async::OS->extract_addrinfo( @_ ) }
-sub unpack_addrinfo  { goto &extract_addrinfo }
-
 =head2 $time = $loop->time
 
 Returns the current UNIX time in fractional seconds. This is currently
@@ -1205,8 +1798,6 @@ sub fork
       _exit( $exitvalue );
    }
 
-   my $loop = $self->{loop};
-
    if( defined $params{on_exit} ) {
       $self->watch_child( $kid => $params{on_exit} );
    }
@@ -1396,6 +1987,8 @@ sub watch_signal
    my $self = shift;
    my ( $signal, $code ) = @_;
 
+   HAVE_SIGNALS or croak "This OS cannot ->watch_signal";
+
    IO::Async::OS->loop_watch_signal( $self, $signal, $code );
 }
 
@@ -1418,6 +2011,8 @@ sub unwatch_signal
    my $self = shift;
    my ( $signal ) = @_;
 
+   HAVE_SIGNALS or croak "This OS cannot ->unwatch_signal";
+
    IO::Async::OS->loop_unwatch_signal( $self, $signal );
 }
 
@@ -1702,6 +2297,28 @@ sub unwatch_idle
    splice @$deferrals, $idx, 1, () if defined $idx;
 }
 
+sub _reap_children
+{
+   my ( $childwatches ) = @_;
+
+   while( 1 ) {
+      my $zid = waitpid( -1, WNOHANG );
+
+      last if !defined $zid or $zid < 1;
+      my $status = $?;
+
+      if( defined $childwatches->{$zid} ) {
+         $childwatches->{$zid}->( $zid, $status );
+         delete $childwatches->{$zid};
+      }
+
+      if( defined $childwatches->{0} ) {
+         $childwatches->{0}->( $zid, $status );
+         # Don't delete it
+      }
+   }
+}
+
 =head2 $loop->watch_child( $pid, $code )
 
 This method adds a new handler for the termination of the given child process
@@ -1747,25 +2364,10 @@ sub watch_child
 
    croak "Already have a handler for $pid" if exists $childwatches->{$pid};
 
-   if( !$self->{childwatch_sigid} ) {
-      $self->{childwatch_sigid} = $self->attach_signal( CHLD => sub {
-         while( 1 ) {
-            my $zid = waitpid( -1, WNOHANG );
-
-            last if !defined $zid or $zid < 1;
-            my $status = $?;
-
-            if( defined $childwatches->{$zid} ) {
-               $childwatches->{$zid}->( $zid, $status );
-               delete $childwatches->{$zid};
-            }
-
-            if( defined $childwatches->{0} ) {
-               $childwatches->{0}->( $zid, $status );
-               # Don't delete it
-            }
-         }
-      } );
+   if( HAVE_SIGNALS and !$self->{childwatch_sigid} ) {
+      $self->{childwatch_sigid} = $self->attach_signal(
+         CHLD => sub { _reap_children( $childwatches ) }
+      );
 
       # There's a chance the child has already exited
       my $zid = waitpid( $pid, WNOHANG );
@@ -1794,7 +2396,7 @@ sub unwatch_child
 
    delete $childwatches->{$pid};
 
-   if( !keys %$childwatches ) {
+   if( HAVE_SIGNALS and !keys %$childwatches ) {
       $self->detach_signal( CHLD => delete $self->{childwatch_sigid} );
    }
 }
@@ -1823,7 +2425,10 @@ sub _adjust_timeout
    $$timeref = 0, return if @{ $self->{deferrals} };
 
    if( defined $self->{sigproxy} and !$params{no_sigwait} ) {
-      $$timeref = $MAX_SIGWAIT_TIME if( !defined $$timeref or $$timeref > $MAX_SIGWAIT_TIME );
+      $$timeref = $MAX_SIGWAIT_TIME if !defined $$timeref or $$timeref > $MAX_SIGWAIT_TIME;
+   }
+   if( !HAVE_SIGNALS and keys %{ $self->{childwatches} } ) {
+      $$timeref = $MAX_CHILDWAIT_TIME if !defined $$timeref or $$timeref > $MAX_CHILDWAIT_TIME;
    }
 
    my $timequeue = $self->{timequeue};
@@ -1869,6 +2474,11 @@ sub _manage_queues
       $count++;
    }
 
+   my $childwatches = $self->{childwatches};
+   if( !HAVE_SIGNALS and keys %$childwatches ) {
+      _reap_children( $childwatches );
+   }
+
    return $count;
 }
 
@@ -1910,6 +2520,52 @@ The following methods take an C<extensions> parameter:
  $loop->connect
  $loop->listen
 
+If an extension C<listen> method is invoked, it will be passed a C<listener>
+parameter even if one was not provided to the original C<< $loop->listen >>
+call, and it will not receive any of the C<on_*> event callbacks. It should
+use the C<acceptor> parameter on the C<listener> object.
+
+=cut
+
+=head1 STALL WATCHDOG
+
+A well-behaved C<IO::Async> program should spend almost all of its time
+blocked on input using the underlying C<IO::Async::Loop> instance. The stall
+watchdog is an optional debugging feature to help detect CPU spinlocks and
+other bugs, where control is not returned to the loop every so often.
+
+If the watchdog is enabled and an event handler consumes more than a given
+amount of real time before returning to the event loop, it will be interrupted
+by printing a stack trace and terminating the program. The watchdog is only in
+effect while the loop itself is not blocking; it won't fail simply because the
+loop instance is waiting for input or timers.
+
+It is implemented using C<SIGALRM>, so if enabled, this signal will no longer
+be available to user code. (Though in any case, most uses of C<alarm()> and
+C<SIGALRM> are better served by one of the L<IO::Async::Timer> subclasses).
+
+The following environment variables control its behaviour.
+
+=over 4
+
+=item IO_ASYNC_WATCHDOG => BOOL
+
+Enables the stall watchdog if set to a non-zero value.
+
+=item IO_ASYNC_WATCHDOG_INTERVAL => INT
+
+Watchdog interval, in seconds, to pass to the C<alarm(2)> call. Defaults to 10
+seconds.
+
+=item IO_ASYNC_WATCHDOG_SIGABRT => BOOL
+
+If enabled, the watchdog signal handler will raise a C<SIGABRT>, which usually
+has the effect of breaking out of a running program in debuggers such as
+F<gdb>. If not set then the process is terminated by throwing an exception with
+C<die>.
+
+=back
+
 =cut
 
 =head1 AUTHOR
