@@ -27,7 +27,7 @@ use POSIX qw( SIGTERM WIFEXITED WEXITSTATUS WIFSIGNALED WTERMSIG );
 use Socket qw( sockaddr_family AF_UNIX );
 use Time::HiRes qw( time );
 
-our $VERSION = '0.51_002';
+our $VERSION = '0.60_001';
 
 # Abstract Units of Time
 use constant AUT => $ENV{TEST_QUICK_TIMERS} ? 0.1 : 1;
@@ -157,7 +157,7 @@ Tests the Loop's ability to watch filehandles for IO readiness
 
 =cut
 
-use constant count_tests_io => 16;
+use constant count_tests_io => 17;
 sub run_tests_io
 {
    {
@@ -191,6 +191,25 @@ sub run_tests_io
       # Ready $S1 to clear the data
       $S1->getline; # ignore return
 
+      $loop->unwatch_io(
+         handle => $S1,
+         on_read_ready => 1,
+      );
+
+      $loop->watch_io(
+         handle => $S1,
+         on_read_ready => sub { $readready = 1 },
+      );
+
+      $readready = 0;
+      $S2->syswrite( "more data\n" );
+
+      $loop->loop_once( 0.1 );
+
+      is( $readready, 1, '$readready after ->unwatch_io/->watch_io' );
+
+      $S1->getline; # ignore return
+
       $loop->watch_io(
          handle => $S1,
          on_write_ready => sub { $writeready = 1 },
@@ -529,6 +548,11 @@ Tests the Loop's ability to watch POSIX signals
 use constant count_tests_signal => 14;
 sub run_tests_signal
 {
+   unless( IO::Async::OS->HAVE_SIGNALS ) {
+      SKIP: { skip "This OS does not have signals", 14; }
+      return;
+   }
+
    my $caught = 0;
 
    $loop->watch_signal( TERM => sub { $caught++ } );
