@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2012-2013 -- leonerd@leonerd.org.uk
 
 package IO::Async::OS;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.51_002';
+our $VERSION = '0.60_001';
 
 our @ISA = qw( IO::Async::OS::_Base );
 
@@ -40,6 +40,8 @@ use constant HAVE_FAKE_ISREG_READY => 0;
 
 # Do we have to select() for for evec to get connect() failures
 use constant HAVE_SELECT_CONNECT_EVEC => 0;
+# Ditto; do we have to poll() for POLLPRI to get connect() failures
+use constant HAVE_POLL_CONNECT_POLLPRI => 0;
 
 # Does connect() yield EWOULDBLOCK for nonblocking in progress?
 use constant HAVE_CONNECT_EWOULDBLOCK => 0;
@@ -47,6 +49,16 @@ use constant HAVE_CONNECT_EWOULDBLOCK => 0;
 # Can we rename() files that are open?
 use constant HAVE_RENAME_OPEN_FILES => 1;
 
+# Do we have IO::Socket::IP available?
+use constant HAVE_IO_SOCKET_IP => defined eval { require IO::Socket::IP };
+
+# Can we reliably watch for POSIX signals, including SIGCHLD to reliably
+# inform us that a fork()ed child has exit()ed?
+use constant HAVE_SIGNALS => 1;
+
+# Preferred trial order for built-in Loop classes
+use constant LOOP_BUILTIN_CLASSES => qw( Poll Select );
+
 =head1 NAME
 
 C<IO::Async::OS> - operating system abstractions for C<IO::Async>
@@ -115,15 +127,21 @@ sub socket
    my ( $family, $socktype, $proto ) = @_;
 
    croak "Cannot create a new socket without a family" unless $family;
+   # PF_UNSPEC and undef are both false
+   $family = $self->getfamilybyname( $family ) || AF_UNIX;
 
    # SOCK_STREAM is the most likely
-   defined $socktype or $socktype = SOCK_STREAM;
+   $socktype = $self->getsocktypebyname( $socktype ) || SOCK_STREAM;
 
    defined $proto or $proto = 0;
 
+   if( HAVE_IO_SOCKET_IP and ( $family == AF_INET || $family == AF_INET6() ) ) {
+      return IO::Socket::IP->new->socket( $family, $socktype, $proto );
+   }
+
    my $sock = eval {
       IO::Socket->new(
-         Domain => $family, 
+         Domain => $family,
          Type   => $socktype,
          Proto  => $proto,
       );
@@ -379,6 +397,8 @@ sub extract_addrinfo
 =item family => 'inet'
 
 Will pack an IP address and port number from keys called C<ip> and C<port>.
+If C<ip> is missing it will be set to "0.0.0.0". If C<port> is missing it will
+be set to 0.
 
 =cut
 
@@ -387,8 +407,8 @@ sub _extract_addrinfo_inet
    my $self = shift;
    my ( $ai ) = @_;
 
-   defined( my $port = $ai->{port} ) or croak "Expected 'port' for extract_addrinfo on family='inet'";
-   defined( my $ip   = $ai->{ip}   ) or croak "Expected 'ip' for extract_addrinfo on family='inet'";
+   my $port = $ai->{port} || 0;
+   my $ip   = $ai->{ip}   || "0.0.0.0";
 
    return pack_sockaddr_in( $port, inet_aton( $ip ) );
 }
@@ -396,8 +416,9 @@ sub _extract_addrinfo_inet
 =item family => 'inet6'
 
 Will pack an IP address and port number from keys called C<ip> and C<port>.
-Optionally will also include values from C<scopeid> and C<flowinfo> keys if
-provided.
+If C<ip> is missing it will be set to "::". If C<port> is missing it will be
+set to 0. Optionally will also include values from C<scopeid> and C<flowinfo>
+keys if provided.
 
 This will only work if a C<pack_sockaddr_in6> function can be found in
 C<Socket>
@@ -409,9 +430,8 @@ sub _extract_addrinfo_inet6
    my $self = shift;
    my ( $ai ) = @_;
 
-   defined( my $port = $ai->{port} ) or croak "Expected 'port' for extract_addrinfo on family='inet6'";
-   defined( my $ip   = $ai->{ip}   ) or croak "Expected 'ip' for extract_addrinfo on family='inet6'";
-
+   my $port     = $ai->{port}     || 0;
+   my $ip       = $ai->{ip}       || "::";
    my $scopeid  = $ai->{scopeid}  || 0;
    my $flowinfo = $ai->{flowinfo} || 0;
 
