@@ -8,7 +8,7 @@ package IO::Async::Protocol;
 use strict;
 use warnings;
 
-our $VERSION = '0.51_002';
+our $VERSION = '0.60_001';
 
 use base qw( IO::Async::Notifier );
 
@@ -227,7 +227,7 @@ sub setup_transport
 
    $transport->configure( 
       on_closed => $self->_capture_weakself( sub {
-         my $self = shift;
+         my $self = shift or return;
          my ( $transport ) = @_;
 
          $self->maybe_invoke_event( on_closed => );
