@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2010-2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2010-2013 -- leonerd@leonerd.org.uk
 
 package IO::Async::Protocol::Stream;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.51_002';
+our $VERSION = '0.60_001';
 
 use base qw( IO::Async::Protocol );
 
@@ -83,7 +83,11 @@ references in parameters:
 
 =head2 $ret = on_read \$buffer, $eof
 
-The C<on_read> handler is invoked identically to C<IO::Async::Stream>.
+=head2 on_read_eof
+
+=head2 on_write_eof
+
+The event handlers are invoked identically to C<IO::Async::Stream>.
 
 =head2 on_closed
 
@@ -101,7 +105,11 @@ The following named parameters may be passed to C<new> or C<configure>:
 
 =item on_read => CODE
 
-CODE reference for the C<on_read> event.
+=item on_read_eof => CODE
+
+=item on_write_eof => CODE
+
+CODE references for the events.
 
 =item handle => IO
 
@@ -119,7 +127,7 @@ sub configure
    my $self = shift;
    my %params = @_;
 
-   for (qw( on_read )) {
+   for (qw( on_read on_read_eof on_write_eof )) {
       $self->{$_} = delete $params{$_} if exists $params{$_};
    }
 
@@ -152,11 +160,17 @@ sub setup_transport
    $self->SUPER::setup_transport( $transport );
 
    $transport->configure( 
-      on_read => $self->_capture_weakself( sub {
-         my $self = shift;
-         my ( $transport, $buffref, $eof ) = @_;
-
-         $self->invoke_event( on_read => $buffref, $eof );
+      on_read => $self->_replace_weakself( sub {
+         my $self = shift or return;
+         $self->invoke_event( on_read => @_ );
+      } ),
+      on_read_eof => $self->_replace_weakself( sub {
+         my $self = shift or return;
+         $self->maybe_invoke_event( on_read_eof => @_ );
+      } ),
+      on_write_eof => $self->_replace_weakself( sub {
+         my $self = shift or return;
+         $self->maybe_invoke_event( on_write_eof => @_ );
       } ),
    );
 }
@@ -197,7 +211,8 @@ sub write
       $args{on_flush} = $self->_replace_weakself( $args{on_flush} );
    }
 
-   $self->transport->write( $data, %args );
+   my $transport = $self->transport or croak "Attempted to ->write to a ".ref($self)." with no transport";
+   $transport->write( $data, %args );
 }
 
 =head2 $protocol->connect( %args )
