@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007-2013 -- leonerd@leonerd.org.uk
 
 package IO::Async::Resolver;
 
@@ -9,9 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Function );
 
-our $VERSION = '0.51_002';
-
-use CPS::Future;
+our $VERSION = '0.60_001';
 
 use Socket 1.93 qw(
    AI_NUMERICHOST AI_PASSIVE
@@ -19,13 +17,6 @@ use Socket 1.93 qw(
    EAI_NONAME
 );
 
-BEGIN {
-   # We're going to implement methods called getaddrinfo and getnameinfo. We
-   # therefore need import them with different names prefixed with underscores
-   *_getaddrinfo = \&Socket::getaddrinfo;
-   *_getnameinfo = \&Socket::getnameinfo;
-}
-
 use IO::Async::OS;
 
 use Time::HiRes qw( alarm );
@@ -49,29 +40,18 @@ This object is used indirectly via an C<IO::Async::Loop>:
  $loop->resolver->getaddrinfo(
     host    => "www.example.com",
     service => "http",
+ )->on_done( sub {
+    foreach my $addr ( @_ ) {
+       printf "http://www.example.com can be reached at " .
+          "socket(%d,%d,%d) + connect('%v02x')\n",
+          @{$addr}{qw( family socktype protocol addr )};
+    }
+ });
 
-    on_resolved => sub {
-       foreach my $addr ( @_ ) {
-          printf "http://www.example.com can be reached at " .
-             "socket(%d,%d,%d) + connect('%v02x')\n",
-             @{$addr}{qw( family socktype protocol addr )};
-       }
-    },
-
-    on_error => sub {
-       print "Cannot look up www.example.com - $_[-1]\n";
-    },
- );
-
- $loop->resolve( type => 'getpwuid', data => [ $< ],
-    on_resolved => sub {
-       print "My passwd ent: " . join( "|", @_ ) . "\n";
-    },
-
-    on_error => sub {
-       print "Cannot look up my passwd ent - $_[-1]\n";
-    },
- );
+ $loop->resolve( type => 'getpwuid', data => [ $< ] )
+    ->on_done( sub {
+    print "My passwd ent: " . join( "|", @_ ) . "\n";
+ });
 
  $loop->run;
 
@@ -131,7 +111,7 @@ sub _init
 
 =cut
 
-=head2 $loop->resolve( %params )
+=head2 $loop->resolve( %params ) ==> @result
 
 Performs a single name resolution operation, as given by the keys in the hash.
 
@@ -154,11 +134,22 @@ specific function chosen by the C<type>; see BUILT-IN RESOLVERS.
 Optional. Timeout in seconds, after which the resolver operation will abort
 with a timeout exception. If not supplied, a default of 10 seconds will apply.
 
+=back
+
+=head2 $resolver->resolve( %params )
+
+When not returning a future, additional parameters can be given containing the
+continuations to invoke on success or failure:
+
+=over 8
+
 =item on_resolved => CODE
 
 A continuation that is invoked when the resolver function returns a successful
 result. It will be passed the array returned by the resolver function.
 
+ $on_resolved->( @result )
+
 =item on_error => CODE
 
 A continuation that is invoked when the resolver function fails. It will be
@@ -166,11 +157,6 @@ passed the exception thrown by the function.
 
 =back
 
-=head2 $task = $resolver->resolve( %params )
-
-When returning a task, the C<on_resolved> and C<on_error> continuations are
-optional.
-
 =cut
 
 sub resolve
@@ -194,7 +180,7 @@ sub resolve
       ref $on_resolved or croak "Expected 'on_resolved' to be a reference";
    }
    elsif( !defined wantarray ) {
-      croak "Expected 'on_resolved' or to return a Task";
+      croak "Expected 'on_resolved' or to return a Future";
    }
 
    my $on_error;
@@ -202,22 +188,22 @@ sub resolve
       ref $on_error or croak "Expected 'on_error' to be a reference";
    }
    elsif( !defined wantarray ) {
-      croak "Expected 'on_error' or to return a Task";
+      croak "Expected 'on_error' or to return a Future";
    }
 
    my $timeout = $args{timeout} || 10;
 
-   my $task = $self->call(
+   my $future = $self->call(
       args => [ $type, $timeout, @{$args{data}} ],
    );
 
-   $task->on_done( $on_resolved ) if $on_resolved;
-   $task->on_fail( $on_error    ) if $on_error;
+   $future->on_done( $on_resolved ) if $on_resolved;
+   $future->on_fail( $on_error    ) if $on_error;
 
-   return $task;
+   return $future;
 }
 
-=head2 $resolver->getaddrinfo( %args )
+=head2 $resolver->getaddrinfo( %args ) ==> @addrs
 
 A shortcut wrapper around the C<getaddrinfo> resolver, taking its arguments in
 a more convenient form.
@@ -253,12 +239,32 @@ C<Socket>.
 
 Time in seconds after which to abort the lookup with a C<Timed out> exception
 
+=back
+
+On success, the future will yield the result as a list of HASH references;
+each containing one result. Each result will contain fields called C<family>,
+C<socktype>, C<protocol> and C<addr>. If requested by C<AI_CANONNAME> then the
+C<canonname> field will also be present.
+
+As a specific optimisation, this method will try to perform a lookup of
+numeric values synchronously, rather than asynchronously, if it looks likely
+to succeed.
+
+Specifically, if the service name is entirely numeric, and the hostname looks
+like an IPv4 or IPv6 string, a synchronous lookup will first be performed
+using the C<AI_NUMERICHOST> flag. If this gives an C<EAI_NONAME> error, then
+the lookup is performed asynchronously instead.
+
+=head2 $resolver->getaddrinfo( %args )
+
+When not returning a future, additional parameters can be given containing the
+continuations to invoke on success or failure:
+
+=over 8
+
 =item on_resolved => CODE
 
-Callback which is invoked after a successful lookup. Will be passed a list of
-HASH references; each containing one result. Each result will contain fields
-called C<family>, C<socktype>, C<protocol> and C<addr>. If requested by
-C<AI_CANONNAME> then the C<canonname> field will also be valid.
+Callback which is invoked after a successful lookup.
 
  $on_resolved->( @addrs )
 
@@ -270,20 +276,6 @@ Callback which is invoked after a failed lookup, including for a timeout.
 
 =back
 
-As a specific optimsation, this method will try to perform a lookup of numeric
-values synchronously, rather than asynchronously, if it looks likely to
-succeed.
-
-Specifically, if the service name is entirely numeric, and the hostname looks
-like an IPv4 or IPv6 string, a synchronous lookup will first be performed
-using the C<AI_NUMERICHOST> flag. If this gives an C<EAI_NONAME> error, then
-the lookup is performed asynchronously instead.
-
-=head2 $task = $resolver->getaddrinfo( %args )
-
-When returning a task, the C<on_resolved> and C<on_error> continuations are
-optional.
-
 =cut
 
 sub getaddrinfo
@@ -292,10 +284,10 @@ sub getaddrinfo
    my %args = @_;
 
    $args{on_resolved} or defined wantarray or
-      croak "Expected 'on_resolved' or to return a Task";
+      croak "Expected 'on_resolved' or to return a Future";
 
    $args{on_error} or defined wantarray or
-      croak "Expected 'on_error' or to return a Task";
+      croak "Expected 'on_error' or to return a Future";
 
    my $host    = $args{host}    || "";
    my $service = $args{service} || "";
@@ -319,28 +311,26 @@ sub getaddrinfo
    if( ( $host =~ m/^[\d.]+$/ or $host =~ m/^[[:xdigit:]:]$/ or $host eq "" ) and
        $service =~ m/^\d+$/ ) {
 
-       my ( $err, @results ) = _getaddrinfo( $host, $service,
+       my ( $err, @results ) = Socket::getaddrinfo( $host, $service,
           { %args, flags => $flags | AI_NUMERICHOST }
        );
 
        if( !$err ) {
-          my $task = CPS::Future->new;
-          $task->on_done( $args{on_resolved} ) if $args{on_resolved};
-          $task->done( @results );
-          return $task;
+          my $future = $self->loop->new_future->done( @results );
+          $future->on_done( $args{on_resolved} ) if $args{on_resolved};
+          return $future;
        }
        elsif( $err == EAI_NONAME ) {
           # fallthrough to async case
        }
        else {
-          my $task = CPS::Future->new;
-          $task->on_fail( $args{on_error} ) if $args{on_error};
-          $task->fail( "$err\n" );
-          return $task;
+          my $future = $self->loop->new_future->fail( $err, resolve => getaddrinfo => $err+0 );
+          $future->on_fail( $args{on_error} ) if $args{on_error};
+          return $future;
        }
    }
 
-   return $self->resolve(
+   my $future = $self->resolve(
       type    => "getaddrinfo_hash",
       data    => [
          host    => $host,
@@ -349,12 +339,18 @@ sub getaddrinfo
          map { exists $args{$_} ? ( $_ => $args{$_} ) : () } qw( family socktype protocol ),
       ],
       timeout => $args{timeout},
-      on_resolved => $args{on_resolved},
-      on_error    => $args{on_error},
-   );
+   )->else( sub {
+      my $message = shift;
+      Future->new->fail( $message, resolve => getaddrinfo => @_ );
+   });
+
+   $future->on_done( $args{on_resolved} ) if $args{on_resolved};
+   $future->on_fail( $args{on_error}    ) if $args{on_error};
+
+   return $future;
 }
 
-=head2 $resolver->getnameinfo( %args )
+=head2 $resolver->getnameinfo( %args ) ==> ( $host, $service )
 
 A shortcut wrapper around the C<getnameinfo> resolver, taking its arguments in
 a more convenient form.
@@ -386,9 +382,22 @@ If true, sets both C<NI_NUMERICHOST> and C<NI_NUMERICSERV> flags.
 
 Time in seconds after which to abort the lookup with a C<Timed out> exception
 
+=back
+
+As a specific optimisation, this method will try to perform a lookup of
+numeric values synchronously, rather than asynchronously, if both the
+C<NI_NUMERICHOST> and C<NI_NUMERICSERV> flags are given.
+
+=head2 $future = $resolver->getnameinfo( %args )
+
+When not returning a future, additional parameters can be given containing the
+continuations to invoke on success or failure:
+
+=over 8
+
 =item on_resolved => CODE
 
-Callback which is invoked after a successful lookup. 
+Callback which is invoked after a successful lookup.
 
  $on_resolved->( $host, $service )
 
@@ -400,15 +409,6 @@ Callback which is invoked after a failed lookup, including for a timeout.
 
 =back
 
-As a specific optimsation, this method will try to perform a lookup of numeric
-values synchronously, rather than asynchronously, if both the
-C<NI_NUMERICHOST> and C<NI_NUMERICSERV> flags are given.
-
-=head2 $task = $resolver->getnameinfo( %args )
-
-When returning a task, the C<on_resolved> and C<on_error> continuations are
-optional.
-
 =cut
 
 sub getnameinfo
@@ -417,10 +417,10 @@ sub getnameinfo
    my %args = @_;
 
    $args{on_resolved} or defined wantarray or
-      croak "Expected 'on_resolved' or to return a Task";
+      croak "Expected 'on_resolved' or to return a Future";
 
    $args{on_error} or defined wantarray or
-      croak "Expected 'on_error' or to return a Task";
+      croak "Expected 'on_error' or to return a Future";
 
    my $flags = $args{flags} || 0;
 
@@ -432,33 +432,35 @@ sub getnameinfo
 
    if( $flags & (NI_NUMERICHOST|NI_NUMERICSERV) ) {
       # This is a numeric-only lookup that can be done synchronously
-      my ( $err, $host, $service ) = _getnameinfo( $args{addr}, $flags );
-      my $task = CPS::Future->new;
+      my ( $err, $host, $service ) = Socket::getnameinfo( $args{addr}, $flags );
 
       if( $err ) {
-         $task->on_fail( $args{on_error} ) if $args{on_error};
-         $task->fail( "$err\n" );
+         my $future = $self->loop->new_future->fail( $err, resolve => getnameinfo => $err+0 );
+         $future->on_fail( $args{on_error} ) if $args{on_error};
+         return $future;
       }
       else {
-         $task->on_done( $args{on_resolved} ) if $args{on_resolved};
-         $task->done( $host, $service );
+         my $future = $self->loop->new_future->done( $host, $service );
+         $future->on_done( $args{on_resolved} ) if $args{on_resolved};
+         return $future;
       }
-
-      return $task;
    }
 
-   my $task = $self->resolve(
+   my $future = $self->resolve(
       type    => "getnameinfo",
       data    => [ $args{addr}, $flags ],
       timeout => $args{timeout},
    )->transform(
       done => sub { @{ $_[0] } }, # unpack the ARRAY ref
-   );
+   )->else( sub {
+      my $message = shift;
+      Future->new->fail( $message, resolve => getnameinfo => @_ );
+   });
 
-   $task->on_done( $args{on_resolved} ) if $args{on_resolved};
-   $task->on_fail( $args{on_error}    ) if $args{on_error};
+   $future->on_done( $args{on_resolved} ) if $args{on_resolved};
+   $future->on_fail( $args{on_error}    ) if $args{on_error};
 
-   return $task;
+   return $future;
 }
 
 =head1 FUNCTIONS
@@ -518,23 +520,23 @@ taking and returning a list of values exactly as the perl function does:
 
 # Now register the inbuilt methods
 
-register_resolver getpwnam => sub { return getpwnam( $_[0] ) or die "$!\n" };
-register_resolver getpwuid => sub { return getpwuid( $_[0] ) or die "$!\n" };
+register_resolver getpwnam => sub { my @r = getpwnam( $_[0] ) or die "$!\n"; @r };
+register_resolver getpwuid => sub { my @r = getpwuid( $_[0] ) or die "$!\n"; @r };
 
-register_resolver getgrnam => sub { return getgrnam( $_[0] ) or die "$!\n" };
-register_resolver getgrgid => sub { return getgrgid( $_[0] ) or die "$!\n" };
+register_resolver getgrnam => sub { my @r = getgrnam( $_[0] ) or die "$!\n"; @r };
+register_resolver getgrgid => sub { my @r = getgrgid( $_[0] ) or die "$!\n"; @r };
 
-register_resolver getservbyname => sub { return getservbyname( $_[0], $_[1] ) or die "$!\n" };
-register_resolver getservbyport => sub { return getservbyport( $_[0], $_[1] ) or die "$!\n" };
+register_resolver getservbyname => sub { my @r = getservbyname( $_[0], $_[1] ) or die "$!\n"; @r };
+register_resolver getservbyport => sub { my @r = getservbyport( $_[0], $_[1] ) or die "$!\n"; @r };
 
-register_resolver gethostbyname => sub { return gethostbyname( $_[0] ) or die "$!\n" };
-register_resolver gethostbyaddr => sub { return gethostbyaddr( $_[0], $_[1] ) or die "$!\n" };
+register_resolver gethostbyname => sub { my @r = gethostbyname( $_[0] ) or die "$!\n"; @r };
+register_resolver gethostbyaddr => sub { my @r = gethostbyaddr( $_[0], $_[1] ) or die "$!\n"; @r };
 
-register_resolver getnetbyname => sub { return getnetbyname( $_[0] ) or die "$!\n" };
-register_resolver getnetbyaddr => sub { return getnetbyaddr( $_[0], $_[1] ) or die "$!\n" };
+register_resolver getnetbyname => sub { my @r = getnetbyname( $_[0] ) or die "$!\n"; @r };
+register_resolver getnetbyaddr => sub { my @r = getnetbyaddr( $_[0], $_[1] ) or die "$!\n"; @r };
 
-register_resolver getprotobyname   => sub { return getprotobyname( $_[0] ) or die "$!\n" };
-register_resolver getprotobynumber => sub { return getprotobynumber( $_[0] ) or die "$!\n" };
+register_resolver getprotobyname   => sub { my @r = getprotobyname( $_[0] ) or die "$!\n"; @r };
+register_resolver getprotobynumber => sub { my @r = getprotobynumber( $_[0] ) or die "$!\n"; @r };
 
 =pod
 
@@ -587,7 +589,7 @@ register_resolver getaddrinfo_hash => sub {
    # Clear any other existing but undefined hints
    defined $args{$_} or delete $args{$_} for keys %args;
 
-   my ( $err, @addrs ) = _getaddrinfo( $host, $service, \%args );
+   my ( $err, @addrs ) = Socket::getaddrinfo( $host, $service, \%args );
 
    die "$err\n" if $err;
 
@@ -606,7 +608,7 @@ register_resolver getaddrinfo_array => sub {
    $hints{protocol} = $protocol if defined $protocol;
    $hints{flags}    = $flags    if defined $flags;
 
-   my ( $err, @addrs ) = _getaddrinfo( $host, $service, \%hints );
+   my ( $err, @addrs ) = Socket::getaddrinfo( $host, $service, \%hints );
 
    die "$err\n" if $err;
 
@@ -619,7 +621,7 @@ register_resolver getaddrinfo_array => sub {
 register_resolver getnameinfo => sub {
    my ( $addr, $flags ) = @_;
 
-   my ( $err, $host, $service ) = _getnameinfo( $addr, $flags || 0 );
+   my ( $err, $host, $service ) = Socket::getnameinfo( $addr, $flags || 0 );
 
    die "$err\n" if $err;
 
