@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Timer );
 
-our $VERSION = '0.51_002';
+our $VERSION = '0.60_001';
 
 use Carp;
 
@@ -221,7 +221,7 @@ sub _make_cb
    my $self = shift;
 
    return $self->_capture_weakself( sub {
-      my $self = shift;
+      my $self = shift or return;
 
       undef $self->{first_interval};
 
@@ -229,7 +229,8 @@ sub _make_cb
 
       $self->invoke_event( on_tick => );
 
-      $self->start;
+      # detect ->stop
+      $self->start if defined $self->{next_time};
    } );
 }
 
